package org.owasp.hashChallenge;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Security;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.provider.JDKMessageDigest;
import org.owasp.lmHash.LMHasher;

/**
 * Class used to produce the hashes for the OWASP AppSec Research 2010 February Challenge
 * http://www.owasp.org/index.php?title=OWASP_AppSec_Research_2010_-_Stockholm,_Sweden#tab=Challenges
 * 
 * Needs Bouncy Castle 1.45 (or later)
 * http://www.bouncycastle.org/latest_releases.html
 * 
 * The hashes to crack:
 * LM(pwd1)                0C04DACA901299DBAAD3B435B51404EE
 * MD2(pwd2 + pwd1)        16189F5462BF906E9D88CF6F152DE86F
 * MD4(pwd3 + pwd2)        FA8F46A6D347087D6980C3FA77DD4DE9
 * MD5(pwd4 + pwd3)        425B33D6F60394C897B8413B5C185845
 * RIPEMD160(pwd5 + pwd4)  35F34671D30472D403937820DCABC1C78C837071
 * SHA1(pwd6 + pwd5)       AE81A30510B2931921934218636B26A803330EB1
 * SHA256(pwd7 + pwd6)     B2FF0269E927C6559804A37590A0688C45DF143F85CEE0E3F239F846B65C9644
 * GOST3411(pwd8 + pwd7)   16CC9F1FF65688E040F5ADA82A41A258FF948769CDA4C4A17D85228A6F358971
 * 
 * Each password comprises of a-zA-Z (mixed alpha) and is max 5 characters long.
 * With salt (i e the previous password) that means max 10 mixed alpha characters
 * as input to the hash function.
 * 
 * Points to earn:
 * pwd1 =>   1 point
 * pwd2 =>   3 points
 * pwd3 =>   5 points
 * pwd4 =>   9 points
 * pwd5 =>  15 points
 * pwd6 =>  25 points
 * pwd7 =>  50 points
 * pwd8 => 100 points
 * 
 * First one to publish the correct password on the sla.ckers.org thread wins the points:
 * http://sla.ckers.org/forum/read.php?11,33533
 * 
 * Please send an email at the same time so we can clear any misunderstandings such as hash
 * collisions:
 * @author john.wilander@owasp.org
 */
public class Hasher {

	private LMHasher lm;
	private MessageDigest md2, md4, md5, sha1, sha256, gost3411, ripeMd160;

	private Hasher() throws NoSuchAlgorithmException {
		Security.addProvider(new BouncyCastleProvider());
		lm = new LMHasher();
		try {
			md2 = JDKMessageDigest.MD2.getInstance("MD2", "BC");
			md4 = JDKMessageDigest.MD4.getInstance("MD4", "BC");
			md5 = JDKMessageDigest.MD5.getInstance("MD5", "BC");
			ripeMd160 = JDKMessageDigest.RIPEMD160.getInstance("RIPEMD160", "BC");
			sha1 = JDKMessageDigest.SHA1.getInstance("SHA1", "BC");
			sha256 = JDKMessageDigest.SHA256.getInstance("SHA256", "BC");
			gost3411 = JDKMessageDigest.GOST3411.getInstance("GOST3411", "BC");
		} catch (NoSuchProviderException e) {
			System.err.println("Bouncy Castle provider not found");
		}
	}
	
	// From http://www.anyexample.com/programming/java/java_simple_class_to_compute_md5_hash.xml
	private String convertToUpperCaseHex(byte[] data) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < data.length; i++) {
            int halfbyte = (data[i] >>> 4) & 0x0F;
            int two_halfs = 0;
            do {
                if ((0 <= halfbyte) && (halfbyte <= 9))
                    buf.append((char) ('0' + halfbyte));
                else
                    buf.append((char) ('a' + (halfbyte - 10)));
                halfbyte = data[i] & 0x0F;
            } while(two_halfs++ < 1);
        }
        return buf.toString().toUpperCase();
    }

	public static void main(String[] args) {
		try {
			Hasher hasher = new Hasher();

			String strToHash = "pwd1";
			System.out.println("LM(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.lm.digest(strToHash)));
			strToHash = "pwd2" + "pwd1";
			System.out.println("MD2(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.md2.digest(strToHash.getBytes())));
			strToHash = "pwd3" + "pwd2";
			System.out.println("MD4(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.md4.digest(strToHash.getBytes())));
			strToHash = "pwd4" + "pwd3";
			System.out.println("MD5(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.md5.digest(strToHash.getBytes())));
			strToHash = "pwd5" + "pwd4";
			System.out.println("RIPEMD160(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.ripeMd160.digest(strToHash.getBytes())));
			strToHash = "pwd6" + "pwd5";
			System.out.println("SHA1(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.sha1.digest(strToHash.getBytes())));
			strToHash = "pwd7" + "pwd6";
			System.out.println("SHA256(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.sha256.digest(strToHash.getBytes())));
			strToHash = "pwd8" + "pwd7";
			System.out.println("GOST3411(\"" + strToHash + "\")\t" + hasher.convertToUpperCaseHex(hasher.gost3411.digest(strToHash.getBytes())));
		} catch (NoSuchAlgorithmException e) {
			System.err.println("Unknown algorithm, " + e.getMessage());
		}
	}
}
